﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Text;
using System.Web;
using Newtonsoft.Json.Linq;
using fds.bestin.util;
using System.Collections.Specialized;
using Newtonsoft.Json;

namespace fds.bestin
{
    public class SOAClient
    {
        private string serverUrl;
        private string webUrl;
        private string appId;
        private string version = "2.0";
        private string dsaPassword, dsaPublicKey, dsaPrivateKey;
        private string rsaPassword, rsaPublicKey, rsaPrivateKey;

        public void setServerUrl(string serverUrl)
        {
            if (serverUrl.LastIndexOf("service/soa") == -1 && serverUrl.EndsWith("/"))
                serverUrl += "service/soa";
            else if (serverUrl.LastIndexOf("service/soa") == -1 && !serverUrl.EndsWith("/"))
                serverUrl += "/service/soa";
            this.serverUrl = serverUrl;
        }

        public void setWebUrl(string webUrl)
        {
            if (webUrl.LastIndexOf("api/view") == -1 && webUrl.EndsWith("/"))
                webUrl += "api/view";
            else if (webUrl.LastIndexOf("api/view") == -1 && !webUrl.EndsWith("/"))
                webUrl += "/api/view";
            this.webUrl = webUrl;
        }

        public void setAppId(string appId)
        {
            this.appId = appId;
        }

        public void setVersion(string version)
        {
            this.version = version;
        }

        public string getRsaPublicKey()
        {
            return this.rsaPublicKey;
        }

        /// <summary>
        /// 设置签名的公私钥
        /// </summary>
        /// <param name="DSAPublicKeyPath">由云平台提供的DSA公钥文件所在目录</param>
        /// <param name="DSAPrivateKeyPath">本地系统生成的DSA私钥文件所在目录</param>
        /// <param name="DSAPrivateKeyPass">本地系统生成的DSA私钥密码</param>
        public void setDSAKey(String DSAPublicKeyPath, String DSAPrivateKeyPath, String DSAPrivateKeyPass)
        {
            dsaPublicKey = DSAPublicKeyPath;
            dsaPrivateKey = DSAPrivateKeyPath;
            dsaPassword = DSAPrivateKeyPass;
        }
        /// <summary>
        /// 设置加密的公私钥
        /// </summary>
        /// <param name="RSAPublicKeyPath">由云平台提供的RSA公钥文件所在目录</param>
        /// <param name="RSAPrivateKeyPath">本地系统生成的RSA私钥文件所在目录</param>
        /// <param name="RSAPrivateKeyPass">本地系统生成的RSA私钥密码</param>
        public void setRSAKey(string RSAPublicKeyPath, string RSAPrivateKeyPath, string RSAPrivateKeyPass)
        {
            rsaPublicKey = RSAPublicKeyPath;
            rsaPrivateKey = RSAPrivateKeyPath;
            rsaPassword = RSAPrivateKeyPass;
        }

        /// <summary>
        /// 获取客户端通知结果并验签
        /// </summary>
        /// <param name="request"></param>
        /// <param name="response"></param>
        /// <returns></returns>
        public Dictionary<string, string> getNotifyResult(HttpRequest request, HttpResponse response)
        {
            NameValueCollection map = request.Params;
            Dictionary<string, string> dict = new Dictionary<string, string>();
            foreach (var k in map.AllKeys)
            {
                if ("".Equals(k))
                    continue;
                dict.Add(k, map[k]);
            }
            StringBuilder sb = new StringBuilder();
            sb.Append(dict["app_id"]).Append(dict["timestamp"]).Append(dict["version"]).Append(dict["event"]);
            string sign = dict["sign"];
            if (sign != null && !"".Equals(sign))
            {
                if (!DSAUtil.verify(sb.ToString(), sign, dsaPublicKey))
                {
                    throw new Exception("签名验证失败");
                }
            }
            return dict;
        }

        /// <summary>
        /// 其他服务调用
        /// </summary>
        /// <param name="service">服务名</param>
        /// <param name="args">参数</param>
        /// <returns></returns>
        public JObject call(string service, Dictionary<string, object> args)
        {
            JObject value = request(service, args);
            return getResult(value);
        }

        private JObject request(string service, Dictionary<string, object> param)
        {
            string strParam = JsonConvert.SerializeObject(JObject.FromObject(param));
            Dictionary<string, string> req = new Dictionary<string, string>();
            req.Add("params", strParam);
            req.Add("app_id", appId);
            req.Add("version", version);
            req.Add("service", service);
            DateTime oldTime = TimeZone.CurrentTimeZone.ToLocalTime(new DateTime(1970, 1, 1));
            double milliseconds = (DateTime.Now - oldTime).TotalMilliseconds;
            string timestamp = Convert.ToString((long)milliseconds / 1000);
            StringBuilder sb = new StringBuilder();
            sb.Append(appId).Append(timestamp).Append(version).Append(service).Append(strParam);
            String sign = DSAUtil.sign(sb.ToString(), dsaPrivateKey, dsaPassword);
            req.Add("timestamp", timestamp);
            req.Add("sign", sign);

            string result = request(req);
            if (result == null)
                throw new Exception("返回数据错误");

            return verifyReturn(JObject.Parse(result));
        }

        public string getWebUrl(string service, Dictionary<string, object> param)
        {
            string strParam = JsonConvert.SerializeObject(JObject.FromObject(param));
            Dictionary<string, string> req = new Dictionary<string, string>();
            req.Add("params", strParam);
            req.Add("app_id", appId);
            req.Add("version", version);
            req.Add("service", service);
            DateTime oldTime = TimeZone.CurrentTimeZone.ToLocalTime(new DateTime(1970, 1, 1));
            double milliseconds = (DateTime.Now - oldTime).TotalMilliseconds;
            string timestamp = Convert.ToString((long)milliseconds / 1000);
            StringBuilder sb = new StringBuilder();
            sb.Append(appId).Append(timestamp).Append(version).Append(service).Append(strParam);
            String sign = DSAUtil.sign(sb.ToString(), dsaPrivateKey, dsaPassword);
            req.Add("timestamp", timestamp);
            req.Add("sign", sign);

            StringBuilder url = new StringBuilder();
            url.Append(webUrl).Append("/").Append(service.Replace(".", "_"));
            int i = 0;
            foreach (KeyValuePair<string, string> item in req)
            {
                if (i == 0)
                    url.Append("?");
                else
                    url.Append("&");
                url.Append(item.Key).Append("=").Append(HttpUtility.UrlEncode(item.Value, Encoding.UTF8));
                i++;
            }

            return url.ToString();
        }

        private JObject verifyReturn(JObject result)
        {
            string response = (string)result["response"];
            string sign = (string)result["sign"];
            if (response != null)
            {
                JObject rel = JObject.Parse(response);
                if (sign != null && !DSAUtil.verify(response, sign, dsaPublicKey))
                    throw new Exception("签名验证错误");
                return rel;
            }

            return result;
        }

        private string request(Dictionary<string, string> param)
        {
            HttpWebRequest req = null;
            HttpWebResponse response = null;
            try
            {
                StringBuilder sb = new StringBuilder();
                sb.Append(serverUrl);
                if (param != null)
                {
                    int i = 0;
                    foreach (KeyValuePair<string, string> item in param)
                    {
                        if (i == 0)
                            sb.Append("?");
                        else
                            sb.Append("&");
                        sb.Append(item.Key).Append("=").Append(HttpUtility.UrlEncode(item.Value, Encoding.UTF8));
                        i++;
                    }
                }
                req = (HttpWebRequest)WebRequest.Create(sb.ToString());
                req.Timeout = 6000;
                req.Headers.Add("pragma", "no-cache");
                req.Headers.Add("Cache-Control", "no-cache");
                req.Headers.Add("Expires", "0");

                response = (HttpWebResponse)req.GetResponse();
                byte[] data = GetBytes(response);
                response.Close();
                return Encoding.UTF8.GetString(data);
            }
            catch (Exception e)
            {
                Debug.Write(e.StackTrace);
            }
            finally
            {
                if (response != null)
                    response.Close();
                if (req != null)
                    req.Abort();
            }
            return null;
        }

        private byte[] GetBytes(WebResponse response)
        {
            var length = (int)response.ContentLength;
            byte[] data;
            using (var memoryStream = new MemoryStream())
            {
                var buffer = new byte[0x100];
                using (var rs = response.GetResponseStream())
                {
                    for (var i = rs.Read(buffer, 0, buffer.Length); i > 0; i = rs.Read(buffer, 0, buffer.Length))
                    {
                        memoryStream.Write(buffer, 0, i);
                    }
                    rs.Close();
                }
                data = memoryStream.ToArray();
                memoryStream.Close();
            }
            return data;
        }
        private JObject getResult(JObject result)
        {
            string status = (string)result["status"];
            if (!"OK".Equals(status, StringComparison.InvariantCultureIgnoreCase))
            {
                string message = (string)result["message"];
                string error_code = (string)result["error_code"];
                string msg = ((message != null) ? message : "") + "=错误码:" + ((error_code != null) ? error_code : "");
                throw new Exception(msg);
            }
            return (JObject)result["result"];
        }

        /// <summary>
        /// 验证签名
        /// </summary>
        /// <param name="text"></param>
        /// <param name="signature"></param>
        /// <returns></returns>
        public bool verify(string text, string signature)
        {
            return DSAUtil.verify(text, signature, dsaPublicKey);
        }
    }
}