﻿using System;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using Org.BouncyCastle.Asn1;
using System.IO;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.X509;

namespace fds.bestin.util
{
    //https://github.com/neoeinstein/bouncycastle
    public class DSAUtil
    {
        /// <summary>
        /// 签名
        /// </summary>
        /// <param name="text"></param>
        /// <param name="privateKey"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public static string sign(string text, string privateKey, string password)
        {
            X509Certificate2 x509Cert = new X509Certificate2(privateKey, password);
            DSACryptoServiceProvider _DSA = (DSACryptoServiceProvider)x509Cert.PrivateKey;

            DSASignatureFormatter DSAFormatter = new DSASignatureFormatter(_DSA);
            DSAFormatter.SetHashAlgorithm("SHA1");

            var sha1 = new SHA1Managed();
            var sha1Hash = sha1.ComputeHash(Encoding.UTF8.GetBytes(text));

            byte[] SignData = DSAFormatter.CreateSignature(sha1Hash);
            SignData = ConvertToDsaSignatureToJavaEncoding(SignData);
            return Convert.ToBase64String(SignData);
        }

        /// <summary>
        /// 验证签名
        /// </summary>
        /// <param name="text"></param>
        /// <param name="signature"></param>
        /// <param name="publicKey"></param>
        /// <returns></returns>
        public static bool verify(string text, string signature, string publicKey)
        {
            X509Certificate2 x509Cert = new X509Certificate2(publicKey);
            var parser = new X509CertificateParser();
            var bouncyCertificate = parser.ReadCertificate(x509Cert.RawData);
            ISigner dsa = SignerUtilities.GetSigner(SignerUtilities.GetObjectIdentifier("DSA"));
            dsa.Init(false, bouncyCertificate.GetPublicKey());
            byte[] input = Encoding.UTF8.GetBytes(text);
            dsa.BlockUpdate(input, 0, input.Length);
            byte[] signatures = Convert.FromBase64String(signature);

            return dsa.VerifySignature(signatures);
        }

        private static byte[] ConvertToDsaSignatureToJavaEncoding(byte[] dsa)
        {
            if (dsa.Length != 40)
                throw new ArgumentException("dsa", "DSA signature should always be 40 bytes long");
            // Split into r and s.
            byte[] r = new byte[20];
            Array.Copy(dsa, 0, r, 0, 20);
            byte[] s = new byte[20];
            Array.Copy(dsa, 20, s, 0, 20);

            // Convert to complement-2
            byte[] complementTwoR = ToComplementTwo(r);
            byte[] complementTwoS = ToComplementTwo(s);

            // Build the result
            byte[] res = new byte[complementTwoR.Length + complementTwoS.Length + 6];
            // Sequence{
            res[0] = 0x30;
            res[1] = (byte)(complementTwoR.Length + complementTwoS.Length + 4);
            // Integer (R)
            res[2] = 0x02;
            res[3] = (byte)complementTwoR.Length;
            Array.Copy(complementTwoR, 0, res, 4, complementTwoR.Length);
            // Integer (S)
            res[complementTwoR.Length + 4] = 0x02;
            res[complementTwoR.Length + 5] = (byte)complementTwoS.Length;
            Array.Copy(complementTwoS, 0, res, complementTwoR.Length + 6, complementTwoS.Length);

            return res;
        }

        private static byte[] ToComplementTwo(byte[] d)
        {
            // Ensure the top-bit is zero, otherwise remove unneeded zeroes
            // - Find non-zero byte
            int i = 0;
            while (i < d.Length && d[i] == 0) i++;
            // - Do we need an extra byte
            int extraByte = (d[i] & 0x80) == 1 ? 1 : 0;
            // - Build the result
            byte[] res = new byte[d.Length - i + extraByte];
            Array.Copy(d, i, res, extraByte, d.Length - i);
            return res;
        }

        private static byte[] ConvertJavaEncodingToDSASignature(byte[] signature)
        {
            Asn1InputStream bIn = new Asn1InputStream(new MemoryStream(signature));
            DerSequence seq = bIn.ReadObject() as DerSequence;

            var r11 = seq[0].GetEncoded();
            var r21 = seq[1].GetEncoded();

            byte[] p1363 = new byte[40];
            Array.Copy(r11, r11.Length - 20, p1363, 0, 20);
            Array.Copy(r21, r21.Length - 20, p1363, 20, 20);

            return p1363;
        }
    }
}